
package threadspack;


import datapack.AccessPointData;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;
import java.util.logging.Level;
import java.util.logging.Logger;
import mainpack.Main;

/**Thread that monitors all the available access points. */
public class AvailableAccessPoints extends Thread
{
    /**List that stores the available access points that are found. */
    private volatile List<AccessPointData> accessPointList = new ArrayList(); 
    /**Name of the wireless interface that is used to scan the environment for available access points. */
    private String wifname;
    /**Boolean that shows if there was a change in the access points list. */
    private boolean changeOccured = false;

    public List<AccessPointData> getAccessPointList()
    {
        return accessPointList;
    }

    public void setAccessPointList(List<AccessPointData> accessPointList)
    {
        this.accessPointList = accessPointList;
    }

    public String getWifname()
    {
        return wifname;
    }

    public void setWifname(String wifname)
    {
        this.wifname = wifname;
    }
   
    /**Scans the environment for available access points and sets the access point data
     * for each one of them.For every access point that finds, creates an AvailableAccessPoints object and 
     * adds it to the list.The data that finds for each object stores them to it and if some 
     * data are not available stores them as "Not-available". */
    public synchronized void scan()
    {  
        String  arg1 = this.wifname;
        //in this list will be added every access point that will be found, to see at the end which of the previous are no more available
        List<String> exists = new ArrayList();
        int i;  
        
        try
        {
            Process c = Runtime.getRuntime().exec("sudo iwlist "+arg1+" scan"); //runs the iwlist command
            try
            {
                c.waitFor();    //waits for the command to finish
            } catch (InterruptedException ex)
            {
                Logger.getLogger(IfDataFinder.class.getName()).log(Level.SEVERE, null, ex);
            }            
            BufferedReader in = new BufferedReader( new InputStreamReader( c.getInputStream()));
            String temps;
            String[] sarray;
            
            temps = in.readLine();
            if (temps == null || temps.contains("No scan results") || temps.contains("Interface doesn't support scanning"))
            {
                this.accessPointList.clear();
                return;
            }
            
            //scan for access points
            temps = in.readLine();
            while ( temps != null)
            {
                if (temps.contains("Cell"))
                { 
                    temps = temps.substring(temps.indexOf("Address:"));
                    sarray = temps.split("\\s+|:\\+");
                    
                    exists.add(sarray[1]);  //adds it to the existanse list                    
                    
                    //check if it is already inserted to the list
                    for ( i = 0; i < this.accessPointList.size(); i++ )
                    {
                        if (this.accessPointList.get(i).getMac_address().equals(sarray[1]) )
                        {
                             break;
                        }                        
                    }
                    AccessPointData ap = new AccessPointData();
                    if ( i != this.accessPointList.size() ) //if it was already inserted
                    {
                        this.accessPointList.set(i, ap);    //refresh its data
                    }
                    else    //if it was not already inserted
                    {
                        this.accessPointList.add(ap);  //add it to the list 
                        this.changeOccured = true;
                    }                    
                    
                    ap.setMac_address(sarray[1]);
                    temps = in.readLine();

                    while ( !temps.contains("Cell") )
                    {
                        if (temps.contains("ESSID:"))
                        {
                            temps = temps.substring(temps.indexOf("ESSID:"));
                            sarray = temps.split(":");
                            ap.setEssid(sarray[1].replace("\"", ""));
                        }

                        if (temps.contains("Mode"))
                        {
                            temps = temps.substring(temps.indexOf("Mode"));
                            sarray = temps.split("\\s+|:|=");
                            ap.setMode(sarray[1]);
                        }

                        if (temps.contains("Channel:") || temps.contains("Channel="))
                        {
                            temps = temps.substring(temps.indexOf("Channel"));
                            sarray = temps.split("\\s+|:|=");
                            ap.setChannel(sarray[1]);
                        }
                        
                        if (ap.getChannel() == null)
                        {
                            if (temps.contains("Frequency"))
                            {
                                temps = temps.substring(temps.indexOf("Frequency"));
                                sarray = temps.split("\\s+|:|=");
                                ap.setChannel(convertChannel(sarray[1]));
                            }
                        }

                        if (temps.contains("Signal level"))
                        {
                            temps = temps.substring(temps.indexOf("Signal level"));
                            sarray = temps.split("\\s+|:|=");
                            ap.setSignal_level(sarray[2].concat(" dBm"));
                        }
                        
                        temps = in.readLine();
                        if ( temps == null )
                        {
                            break;
                        }
                    }                    
                    if (ap.getSignal_level() == null)
                    {
                        ap.setSignal_level("Not-available");
                    }
                }
            }
            in.close();
            
            //check if there where any access points that are no more available
            for ( i = 0; i < this.accessPointList.size(); i++ )
            {
                if ( !exists.contains(this.accessPointList.get(i).getMac_address()) )   //it is ot in the existance list
                {
                    this.accessPointList.remove(i); //remove it fro the list
                    this.changeOccured = true;
                }
            }
                    
        } catch (IOException ex)
        {
            Logger.getLogger(AvailableInterfaces.class.getName()).log(Level.SEVERE, null, ex);
            System.exit(1);
        }
    }  
    
    
    private String convertChannel(String channel)
    {
        int ch = Integer.parseInt(channel.replace(".", ""));
        int convch = ((ch - 2412) / 5) + 1;
        return Integer.toString(convch);
    } 
    
    /**This method is used to print all data of the available access points that this thread has found. */
    public synchronized void printApData()
    {
        System.out.println("\nAvailable Access Points: ");
        if ( this.accessPointList.isEmpty() )
        {
            System.out.println("No scan results");
        }
        else
        {
            for ( int i = 0; i < this.accessPointList.size(); i++)
            {  
                System.out.println("\nCell " + (i + 1) + ":");
                System.out.println(this.accessPointList.get(i).toString());
            }           
        }        
    }     
    
    /**Sets the name of the wireless interface that this thread 
     * will, at first, use to monitor the environment for access points
     * and then starts running the thread.
     * @param wifname the name of the wireless interface to be used*/
    public void startThread( String wifname )
    {
        this.wifname = wifname;
        this.start();        
    }
    
    @Override
    public void run()
    {   
        long t;
        int s = 0;
        ListIterator<Integer> it = Main.markovList.listIterator();         
        
        while ( true )
        {
            try
            { 
                t = System.currentTimeMillis();
                
                this.scan(); 
                
                t = System.currentTimeMillis() - t;

                if ( changeOccured )    //it had an event
                {
                    s = Main.markovList.get(0); //go to the first state
                    it = Main.markovList.listIterator(1);
                    System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+ this.getName()+
                    "] INFO WirelessEnviromentMonitor – Change on the Available Access Points List. State= "+s+" Sleeptime= "+(t > s ? 0 : s - t)+"ms");                     
                }
                else
                {
                    if (it.hasNext())   //if it is not in the end of the markov list
                    {
                        s = it.next();  //go to the next node of the markov list
                    }
                    System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+ this.getName()+
                    "] INFO WirelessEnviromentMonitor – No change on the Available Access Points List. State= "+s+" Sleeptime= "+(t > s ? 0 : s - t)+"ms");                    
                } 
                this.changeOccured = false;
                
                if ( !Main.threadsTable.get("WirelessMonitor").getState() ) //if it receives a termination signal 
                {                  
                    break;
                }                                               
               
               sleep( t > s ? 0 : s-t);
               
                if ( !Main.threadsTable.get("WirelessMonitor").getState() ) //if it receives a termination signal 
                {                  
                    break;
                }                 
                
            } catch (InterruptedException ex)
            {
                Logger.getLogger(AvailableInterfaces.class.getName()).log(Level.SEVERE, null, ex);
                System.exit(1);
            }
        }
        System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+this.getName()
                           +"] INFO AvailableInterfacesMonitor – WirelessEnvironmentMonitor terminated");        
    }        
}
