
package threadspack;


import datapack.InterfaceData;
import datapack.ThreadData;
import datapack.WirelessData;
import java.io.File;
import java.io.FileNotFoundException;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Scanner;
import java.util.logging.Level;
import java.util.logging.Logger;
import mainpack.Main;


/**Thread that monitors all the available network interfaces. */
public class AvailableInterfaces extends Thread
{  
    /**List that stores the available interfaces that are found. */
    private volatile List<InterfaceData> interfaceList = new ArrayList();
    /**List that stores the available wireless interfaces that are found. */
    private volatile List<WirelessData> wirelessList = new ArrayList();
    /**AvailableAccessPoints object that is the thread searching the environment for access points. */
    private volatile AvailableAccessPoints aap = new AvailableAccessPoints();
    /**Boolean that shows if there was a change on any of the network interfaces lists. */
    private boolean changeOccured = false;

    public List<InterfaceData> getInterfaceList()
    {
        return interfaceList;
    }

    public void setInterfaceList(List<InterfaceData> interfaceList)
    {
        this.interfaceList = interfaceList;
    }

    public List<WirelessData> getWirelessList()
    {
        return wirelessList;
    }

    public void setWirelessList(List<WirelessData> wirelessList)
    {
        this.wirelessList = wirelessList;
    }
    
    /**Scans the system for the available network interfaces.For
     * every new interface that founds begins a new thread to monitor it, and 
     * for these that have been disconnected stops the threads that monitors them. */
    public synchronized void scan()
    {
        Scanner input = null;
        
        try
        {
            input = new Scanner(new File("/proc/net/dev")).useDelimiter(":");
        } catch (FileNotFoundException ex)
        {
            Logger.getLogger(AvailableInterfaces.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        Enumeration<NetworkInterface> nets = null;
        try {
            nets = NetworkInterface.getNetworkInterfaces();
        } catch (SocketException ex) {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
        }   
        
        String s;
        //in this list will be added every interface that will be found, to see at the end which intrfaces where disconnected
        List<String> exists = new ArrayList();  
        
        input.nextLine();
        input.nextLine();
        
        //scan for interfaces
        while(input.hasNextLine())
        {
            s = input.next();
            s = s.trim();  
            
            exists.add(s);  //adds it to the existanse list
            
            if( isWireless(s) )
            {
                //if it founds a wireless interface
                if ( !Main.threadsTable.containsKey(s) || //if it doesnt exist
                     ( Main.threadsTable.containsKey(s) && !Main.threadsTable.get(s).getState() ) && //if sometime existed but has been stoped
                      //if we have more than one wireless and it is not the one that scans access points
                     !aap.getWifname().equals(s) 
                   )
                {                
                    WirelessData winterf = new WirelessData(s);                     
                    this.wirelessList.add(winterf); //adds the wireless instance to the list
                    this.changeOccured = true;

                    WifDataFinder wifdf = new WifDataFinder();  //creates the thread that will monitor this wireless interface 

                    wifdf.startThread(s, winterf);
                    ThreadData w_thread = new ThreadData(wifdf, true);  //creates the ThreadData for this thread
                    Main.threadsTable.put(s, w_thread); //adds it to the threads table
                    
                    System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+ wifdf.getName()
                                       +"] INFO AvailableInterfacesMonitor – Started WirelessMonitor for: "+s);
                }                
            }
            else
            {
                //if it founds an interface
                if ( !Main.threadsTable.containsKey(s) || //if it doesnt exist
                     ( Main.threadsTable.containsKey(s) && !Main.threadsTable.get(s).getState() )   //if sometime existed but has been stoped
                   )
                {                
                    InterfaceData interf = new InterfaceData(s);    //adds the interface instance to the list                   
                    this.interfaceList.add(interf);
                    this.changeOccured = true;

                    IfDataFinder ifdf = new IfDataFinder();     //creates the thread that will monitor this interface                    

                    ifdf.startThread(s, interf);
                    ThreadData i_thread = new ThreadData(ifdf, true);   //creates the ThreadData for this thread
                    Main.threadsTable.put(s, i_thread); //adds it to the threads table
                    
                    System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+ ifdf.getName()
                                       +"] INFO AvailableInterfacesMonitor – Started InterfaceMonitor for: "+s);                    
                }
            }                                            
            input.nextLine();
        }      
        input.close();
        
        //scan for virtual interfaces
        for (NetworkInterface netIf : Collections.list(nets)) 
        {
            Enumeration<NetworkInterface> subIfs = netIf.getSubInterfaces();
            for (NetworkInterface subIf : Collections.list(subIfs)) {
                if ( !Main.threadsTable.containsKey(subIf.getName()) || //if it doesnt exist 
                     //if sometime existed but has been stoped
                     ( Main.threadsTable.containsKey(subIf.getName()) && !Main.threadsTable.get(subIf.getName()).getState() )
                   )
                {
                        InterfaceData vinterf = new InterfaceData(subIf.getName()); //adds the virtual interface instance to the list                                    
                        this.interfaceList.add(vinterf);
                        this.changeOccured = true;

                        IfDataFinder vifdf = new IfDataFinder();    //creates the thread that will monitor this interface

                        vifdf.startThread(subIf.getName(), vinterf);
                        ThreadData vi_thread = new ThreadData(vifdf, true); //creates the ThreadData for this thread
                        Main.threadsTable.put(subIf.getName(), vi_thread);  //adds it to the threads table

                        exists.add(subIf.getName());    //adds it to the existanse list

                        System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+ vifdf.getName()
                                           +"] INFO AvailableInterfacesMonitor – Started InterfaceMonitor for: "+subIf.getName());
                }
                else
                {
                    exists.add(subIf.getName());
                }
            }

        }         
        
        int i;
        //check if there where any disconnected interfaces
        for (Map.Entry<String, ThreadData> entry : Main.threadsTable.entrySet())
        {
            if ( !exists.contains(entry.getKey()) &&    //if for the current thread the interface that monitors does not exists
                 !entry.getKey().equals("MasterThread") &&  //it is not the MasterThread
                 !entry.getKey().equals("WirelessMonitor") &&   //it is not the "WirelessMonitor"
                 !entry.getKey().equals("DataPrinter") &&   //it is not the "DataPrinter"
                 entry.getValue().getState()    //it is running                 
                ) 
            {                
                entry.getValue().setState(false);   //sends termination signal
                try
                {
                    entry.getValue().getThread().join();    //waits for it to end

//                    System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+ entry.getValue().getThread().getName()
//                                       +"] INFO AvailableInterfacesMonitor – Terminated InterfaceMonitor for: "+entry.getKey());

                } catch (InterruptedException ex)
                {
                    Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
                }
                
                this.changeOccured = true;
                
                //find it and remove it from the list that contains it 
                for ( i = 0; i < this.interfaceList.size(); i++)
                {
                    if ( this.interfaceList.get(i).getName().equals(entry.getKey()))
                    {
                        this.interfaceList.remove(i);
                        break;
                    }
                }
                if ( i ==  this.interfaceList.size())
                {
                    for ( i = 0; i < this.wirelessList.size(); i++)
                    {
                        if ( this.wirelessList.get(i).getName().equals(entry.getKey()))
                        {
                            this.wirelessList.remove(i);
                            break;
                        }
                    }    
                }
            }
            //if there was a second wireless interface scanning and has disconnected
            if ( !exists.contains(entry.getKey()) && !entry.getValue().getState() && this.aap.getWifname().equals(entry.getKey()) )
            {
                this.changeOccured = true;
                this.wirelessList.remove(1);    //remove from the wireless list
            }
        }

    }
    
    /**Checks if the name of the interface that was given is wireless or not.
     * @param s The name of the interface we want to check
     * @return true if it is wireless, false if it is not wireless*/
    private boolean isWireless(String s)
    {  
        Scanner input = null;
        
        try
        {
            input = new Scanner(new File("/proc/net/wireless")).useDelimiter(":");
        } catch (FileNotFoundException ex)
        {
            Logger.getLogger(AvailableInterfaces.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        input.nextLine(); 
        input.nextLine();
        
        if ( s.contains("wlan"))
        {
            return true;
        }
        
        while(input.hasNextLine())
        {            
            if ( s.equals(input.next().trim()))
            {           
                return true;
            }
            else
            {
                input.nextLine();
            }
        }
        input.close();  
        
        return false;             
    }    
    
    /**Starts running this thread. */
    public void startThread()
    {
        this.start();
    }    
    
    @Override
    public void run()
    {  
        long t, s = 0;                
        int wls;
        ListIterator<Integer> it = Main.markovList.listIterator();

        while ( true )
        {
            try
            {
                t = System.currentTimeMillis();
                
                wls = this.wirelessList.size();
                
                this.scan();
                
                t = System.currentTimeMillis() - t;
                
                if ( this.wirelessList.size() == 1 && wls != this.wirelessList.size() ) //if there was a change and we have one wireless interface
                {
                    System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+ this.aap.getName()+
                                       "] ERROR AvailableInterfacesMonitor – Found 1 wireless interface. Successful execution requires at least 2. "
                                       +"\n\tExecution will continue yet results are not guaranteed.");                    
                    if ( !Main.threadsTable.containsKey("WirelessMonitor") )    //if the wireless environment thread has not started yet
                    {
                        this.aap.startThread( this.wirelessList.get(0).getName() ); //starts it using this wireless interface
                        ThreadData aap_thread = new ThreadData(this.aap, true);     //creates the ThreadData for this thread
                        Main.threadsTable.put("WirelessMonitor", aap_thread);   //adds it to the threads table

                        System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+ this.aap.getName()
                                           +"] INFO AvailableInterfacesMonitor – WirelessEnvironmentMonitor started");
                    }
                    else    //if it has started
                    {                      
                        this.aap.setWifname(this.wirelessList.get(0).getName());    //use this wireless interface to scan from now on
                    }
                }
                if ( this.wirelessList.size() > 1 && wls < this.wirelessList.size()) //if there are more than one wireless and we had an addition 
                { 
                    System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+ this.aap.getName()+
                                       "] INFO AvailableInterfacesMonitor – Found more than 1 wireless interfaces."
                                       +"Execution results will be optimal"); 
                    
                    
                    if ( !Main.threadsTable.containsKey("WirelessMonitor") )    //if the wireless environment thread has not started yet
                    {
                        this.aap.startThread( this.wirelessList.get(1).getName() ); //starts it using this wireless interface
                        ThreadData aap_thread = new ThreadData(this.aap, true);     //creates the ThreadData for this thread
                        Main.threadsTable.put("WirelessMonitor", aap_thread);   //adds it to the threads table

                        System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+ this.aap.getName()
                                           +"] INFO AvailableInterfacesMonitor – WirelessEnvironmentMonitor started");
                    }
                    else    //if it has started
                    {
                        this.aap.setWifname(this.wirelessList.get(1).getName());    //use this wireless interface to scan from now on
                    }
                    
                    Main.threadsTable.get(this.wirelessList.get(1).getName()).setState(false);  //terminate the monitor thread for this interface
                    try
                    {
                        Main.threadsTable.get(this.wirelessList.get(1).getName()).getThread().join(); //waits for it to end

                    } catch (InterruptedException ex)
                    {
                        Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }              
                
                if ( changeOccured )    //it had an event
                {
                    s = Main.markovList.get(0); //go to the first state
                    it = Main.markovList.listIterator(1); 
                    System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+ this.getName()+
                    "] INFO AvailableInterfacesMonitor – Change on the Available Interfaces Lists. State= "+s+" Sleeptime= "+(t > s ? 0 : s - t)+"ms");
                }
                else
                {
                    if (it.hasNext())   //if it is not in the end of the markov list
                    {
                        s = it.next();  //go to the next node of the markov list 
                    }
                    System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+ this.getName()+
                    "] INFO AvailableInterfacesMonitor – No change on the Available Interfaces Lists. State= "+s+" Sleeptime= "+(t > s ? 0 : s - t)+"ms");                    
                }
                this.changeOccured = false;

                if (!Main.threadsTable.get("MasterThread").getState())  //if it receives a termination signal 
                {
                    Main.threadsTable.get("WirelessMonitor").setState(false);   //sends to wireless evironment monitor termination signal
                    
                    for (Map.Entry<String, ThreadData> entry : Main.threadsTable.entrySet())
                    {
                        if ( entry.getValue().getState() )  //if thread running
                        {
                            entry.getValue().setState(false);    //sends thread termination signal                           
                        }
                    }  
                    for (Map.Entry<String, ThreadData> entry : Main.threadsTable.entrySet())
                    {
                        if ( entry.getValue().getThread().isAlive() && !entry.getKey().equals("MasterThread") ){
                            try
                            {
                                entry.getValue().getThread().join();    //waits for them to stop before it exits

                            } catch (InterruptedException ex)
                            {
                                Logger.getLogger(AvailableInterfaces.class.getName()).log(Level.SEVERE, null, ex);
                                System.exit(1);
                            }
                        }
                    } 

                    break;
                }  
                
                sleep(t > s ? 0 : s - t);               

            } catch (InterruptedException ex)
            {
                Logger.getLogger(AvailableInterfaces.class.getName()).log(Level.SEVERE, null, ex);
                System.exit(1);
            }
        }
    }
}
