
package threadspack;


import datapack.InterfaceData;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ListIterator;
import java.util.logging.Level;
import java.util.logging.Logger;
import mainpack.Main;


/**Thread that monitors the data of a specific network interface. */
public class IfDataFinder extends Thread
{  
    /**The name of the monitored interface. */
    private String ifname;
    /**The InterfaceData object of the monitored interface. */
    private InterfaceData interf;
    /**Variables for Received packets, Transmitted packets, Time in milliseconds. */
    private long rxArx, txArx, tArx; 
    
    /**Sets the interface data.For every data that finds
     * stores them to the InterfaceData object of this thread and if some 
     * data are not available stores them as "Not-available". */
    public synchronized void setInterface()
    {
        String  arg1 = this.ifname;
        
        try
        {
            Process c = Runtime.getRuntime().exec("sudo ifconfig "+arg1);   //runs the ifconfig command
            try
            {
                c.waitFor();    //waits for the command to finish
            } catch (InterruptedException ex)
            {
                Logger.getLogger(IfDataFinder.class.getName()).log(Level.SEVERE, null, ex);
            }
            BufferedReader in = new BufferedReader( new InputStreamReader( c.getInputStream()));
            String temps;
            
            temps = in.readLine();             
            this.interf.setMac_address(getMac(temps));
            
            temps = in.readLine();
            setNetData(temps, this.interf);
            
            //this if is when the wireless calls this method and has already set the Max_bitrate
            if ( this.interf.getMax_bitrate() == null ) 
            {
                this.interf.setMax_bitrate(getMBR(arg1)); 
            }
            
            setNetInfo(temps, this.interf, in);
            
            in.close();
        } catch (IOException ex)
        {
            Logger.getLogger(AvailableInterfaces.class.getName()).log(Level.SEVERE, null, ex);
            System.exit(1);
        }
    }
    
    private String getMac(String temps) 
    {
        int i;
        if ( (i = temps.indexOf("HWaddr")) != -1 )
        {
            return (temps.substring(i+7).trim());
        }
        else
        {
            return "Not-available";
        }
    }

    
    private void setNetData(String temps, InterfaceData interf) 
    {   
        int i, j;
        if ( interf.getMac_address().equals("Not-available") )
        {                     
            if ( (i = temps.indexOf("inet addr:")) != -1 )
            {
                interf.setIp_address(temps.substring(i+10, j = temps.indexOf("Mask:")-1).trim());
                interf.setSubnetmask(temps.substring(j+6).trim());
                interf.setNetwork_address(getNetAddr(interf.getIp_address(), interf.getSubnetmask()));
                interf.setBroadcast_address(getBcast(interf.getNetwork_address(), interf.getSubnetmask()));
                interf.setDefault_gateway(getGateway(interf.getName()));
            }
            else
            {
                interf.setIp_address("Not-available");                  
                interf.setSubnetmask("Not-available");                    
                interf.setNetwork_address("Not-available");
                interf.setBroadcast_address("Not-available");
                interf.setDefault_gateway("Not-available");
            }                               
        }
        else
        {
            if ( (i = temps.indexOf("inet addr:")) != -1 )
            {
                interf.setIp_address(temps.substring(i+10, j = temps.indexOf("Bcast:")-1).trim());
                interf.setBroadcast_address(temps.substring(j+7, i = temps.indexOf("Mask:")-1).trim());                    
                interf.setSubnetmask(temps.substring(i+6).trim());
                interf.setNetwork_address(getNetAddr(interf.getIp_address(), interf.getSubnetmask()));
                interf.setDefault_gateway(getGateway(interf.getName()));                    
            }
            else
            {
                interf.setIp_address("Not-available");
                interf.setBroadcast_address("Not-available");
                interf.setSubnetmask("Not-available");
                interf.setNetwork_address("Not-available");
                interf.setDefault_gateway("Not-available");
            }                
        } 
    }   
    
    
    private String getGateway(String ifname)
    {    
        String temps=null;
        
        try
        {
            Process c = Runtime.getRuntime().exec("route -n");  //runs the route command
            try
            {
                c.waitFor();    //waits for the command to finish
            } catch (InterruptedException ex)
            {
                Logger.getLogger(IfDataFinder.class.getName()).log(Level.SEVERE, null, ex);
            }            
            BufferedReader in = new BufferedReader( new InputStreamReader( c.getInputStream()));           
            while(( temps = in.readLine()) != null)
            {       
                if ( temps.startsWith("0.0.0.0") && temps.endsWith(ifname))
                {
                    break;
                }
            } 
            in.close();
        } catch (IOException ex)
        {
            Logger.getLogger(AvailableInterfaces.class.getName()).log(Level.SEVERE, null, ex);
            System.exit(1);
        }
        
        if ( temps != null )
        {
            return temps.substring(temps.indexOf("0.0.0.0")+7, temps.indexOf("0.0.0.0", 8)).trim().toString();
        }
        else
        {
            return "Not-available";
        }
    }
    

    private String getNetAddr( String ip, String mask) 
    {
        String[] iparray = ip.split("\\.");
        String[] maskarray = mask.split("\\.");
        String netaddr = "";

        for ( int i = 0; i < 4; i++)
        {
            netaddr += Integer.parseInt(iparray[i])&Integer.parseInt(maskarray[i]); //AND ip with mask to find network address
            if ( i != 3)
            {
                netaddr += ".";
            }
        }

        return netaddr;
    }
    

    private String getBcast( String netaddr, String mask ) 
    {
        String[] netarray = netaddr.split("\\.");
        String[] maskarray = mask.split("\\.");        
        String bcast="";
        
        mask = "";
        for ( int i = 0; i < 4; i++)
        {
            mask += Integer.parseInt(maskarray[i])^255;  //XOR with 255 to invert subnet mask
            if ( i != 3)
            {
                mask += ".";
            }
        }
        
        maskarray = mask.split("\\.");  //splits the inverted subne mask      
        for ( int i = 0; i < 4; i++)
        {
            bcast += Integer.parseInt(netarray[i])|Integer.parseInt(maskarray[i]);    //OR network address and inverted mask to find broadcast
            if ( i != 3)
            {
                bcast += ".";
            }
        }
        
        return bcast;
    }
    
    
    private String getMBR(String ifname)
    {
        String temps;
        String[] sarray;
        
        try
        {
            Process c = Runtime.getRuntime().exec("ethtool "+ifname);   //runs the ethtool command
            try
            {
                c.waitFor();    //waits for the command to finish
            } catch (InterruptedException ex)
            {
                Logger.getLogger(IfDataFinder.class.getName()).log(Level.SEVERE, null, ex);
            }            
            BufferedReader in = new BufferedReader( new InputStreamReader( c.getInputStream()));           
            while(( temps = in.readLine()) != null)
            {       
                if ( temps.contains("Speed") )
                {
                    sarray = temps.split("\\s+");
                    in.close();
                    if ( sarray[2].contains("Mb/s") )
                    {                        
                        return sarray[2].replace("Mb/s", "").concat("000").concat(" kb/s");
                    }                    
                    return sarray[2].replace("Kb/s", " kb/s");
                }
            } 
            in.close();
        } catch (IOException ex)
        {
            Logger.getLogger(AvailableInterfaces.class.getName()).log(Level.SEVERE, null, ex);
            System.exit(1);
        }  
        
        if (Main.prop.containsKey(ifname)) 
        {
            return Main.prop.getProperty(ifname).concat(" kb/s");
        }
        else
        {
            return "Not-available";
        }        
    }
    
    
    private void setNetInfo(String temps, InterfaceData interf, BufferedReader in) 
    {
        try {
            double rxpacks=0, rxerr=0, txpacks=0, txerr=0;            
            while ( (temps = in.readLine()) != null )
            {
                if ( temps.contains("RX packets:"))
                {
                    String[] sarray = temps.split("\\s+|:");
                    rxpacks = Double.parseDouble(sarray[3]);
                    rxerr = Double.parseDouble(sarray[5]);
                }
                if ( temps.contains("TX packets:"))
                {
                    String[] sarray = temps.split("\\s+|:");
                    txpacks = Double.parseDouble(sarray[3]);
                    txerr = Double.parseDouble(sarray[5]);
                    break;
                }
            }
            if ( temps == null)
            {
                interf.setCurrent_bitrate("Not-available");
                interf.setPacket_error_rate("Not-available");
                interf.setCurrent_usedrange("Not-available");                 
                return;
            } 
            else
            {
                if (rxpacks+txpacks == 0)
                {
                    interf.setPacket_error_rate("0");
                }
                else
                {
                    interf.setPacket_error_rate(Double.toString(((rxerr+txerr)/(rxpacks+txpacks))*100).concat(" %"));
                }

            }          

            while ( (temps = in.readLine()) != null )
            {
                if ( temps.contains("RX bytes:"))
                {
                    interf.setCurrent_bitrate(getCurrBitRate( temps));
                    break;
                }
            }

            if (interf.getMax_bitrate().equals("Not-available"))
            {
                interf.setCurrent_usedrange("Not-available");
            }
            else
            {
            double r = (Double.parseDouble(interf.getCurrent_bitrate().replaceFirst(" kb/s", ""))/
                                        Double.parseDouble(interf.getMax_bitrate().replaceFirst(" kb/s", "")))
                                        *100.0;
            interf.setCurrent_usedrange(Double.toString(r).concat(" %"));
            }  
            
        } catch (IOException ex) {
            Logger.getLogger(IfDataFinder.class.getName()).log(Level.SEVERE, null, ex);
            System.exit(1);
        }
    }    
    

    private String getCurrBitRate(String temps) 
    {
        long rxTel, txTel, tTel;
        String currbitrate;
       
        rxTel = Long.parseLong(temps.substring(temps.indexOf("RX bytes:")+9, temps.indexOf("(")-1));
        txTel = Long.parseLong(temps.substring(temps.indexOf("TX bytes:")+9, temps.indexOf("(", temps.indexOf("TX bytes:")) -1)); 
        
        tTel = System.currentTimeMillis();        
                                         
        if ( this.rxArx == 0 || this.txArx == 0 || this.tArx == 0 )
        {
            this.rxArx = rxTel;
            this.txArx = txTel;
            this.tArx = tTel;
            return "0".concat(" kb/s");
        }
        else
        {
            currbitrate = Long.toString((long)((((rxTel+txTel)-(this.rxArx+this.txArx))*8/1000)/((tTel-this.tArx)/ 1000.0))).concat(" kb/s");         
            this.rxArx = rxTel;
            this.txArx = txTel;
            this.tArx = tTel;            
            return currbitrate;
        }       
    }   
    
    /**This method is used by a wireless interface to set the data of the inherited InterfaceData.
     * @param ifname the name of the interface to be set
     * @param interf the InterfaceData object to be set*/
    public void setWireless(String  ifname, InterfaceData interf)
    {
        this.ifname = ifname;
        this.interf = interf;
        this.setInterface();
    }
    
    /**This method is used to print all data of the interface that is being monitored by this thread.
     * @return all the interface data to a string*/
    public synchronized String getDataString()
    {
        return this.interf.toString();
    }
    
    /**Sets the name and the InterfaceData object of the interface that this thread 
     * will monitor and then starts running the thread.
     * @param ifname the name of the interface to be monitored
     * @param interf the InterfaceData object to be monitored*/
    public void startThread(String  ifname, InterfaceData interf)
    {
        this.ifname = ifname;
        this.interf = interf;
        this.start();
    }    
    
    @Override
    public void run()
    {    
        long t;
        int s = 0;
        ListIterator<Integer> it = Main.markovList.listIterator();

        InterfaceData ifDataTemp;               
        
        while ( true )
        {
            try
            {  
                t = System.currentTimeMillis();
                
                ifDataTemp = (InterfaceData) this.interf.clone();   //clone the current InterfaceData object for future comparison
                
                this.setInterface();
                
                t = System.currentTimeMillis() - t;
                
                //compares the old an the new InterfaceData objects 
                if ( !this.interf.equals(ifDataTemp) )  //it had an event 
                {
                    s = Main.markovList.get(0); //go to the first state
                    it = Main.markovList.listIterator(1); 
                    System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+ this.getName()+
                    "] INFO InterfaceDataFinder – Change on the interface: "+this.ifname+". State= "+s+" Sleeptime= "+(t > s ? 0 : s - t)+"ms");                    
                }
                else
                {
                    if (it.hasNext())   //if it is not in the end of the markov list
                    {
                        s = it.next();  //go to the next node of the markov list 
                    }
                    System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+ this.getName()+
                    "] INFO InterfaceDataFinder – No change on the interface: "+this.ifname+". State= "+s+" Sleeptime= "+(t > s ? 0 : s - t)+"ms");                    
                }                    
                
                if ( !Main.threadsTable.get(ifname).getState() )    //if it receives a termination signal 
                {                    
                    break;
                }                               
               
               sleep( t > s ? 0 : s-t);
               
                if ( !Main.threadsTable.get(ifname).getState() )    //if it receives a termination signal 
                {                    
                    break;
                }                 
                
            } catch (InterruptedException ex)
            {
                Logger.getLogger(AvailableInterfaces.class.getName()).log(Level.SEVERE, null, ex);
                System.exit(1);
            }
        }  
        System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+this.getName()
           +"] INFO AvailableInterfacesMonitor – Terminated InterfaceMonitor for: "+this.ifname); 
    }
    
}
