package threadspack;

import datapack.AccessPointData;
import datapack.InterfaceData;
import datapack.WirelessData;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import mainpack.Main;

/**
 * Thread that periodically sends all the data collected for the interfaces,
 * the wireless interfaces and the access points by the other threads to the web service.
 */
public class MonitorDataBuilder extends Thread
{

    private server.MonitorData MD = new server.MonitorData();
    private List<InterfaceData> interfaceList = new ArrayList();
    private List<WirelessData> wirelessList = new ArrayList(); 
    private List<AccessPointData> accessPointList = new ArrayList(); 
    AvailableAccessPoints aap;

    synchronized private void buildMonitorData()
    {
        if ( !this.interfaceList.isEmpty() )
        {
            this.interfaceList.clear();
        }

        if ( !this.wirelessList.isEmpty() )
        {
            this.wirelessList.clear();
        }    
            
        if ( !this.accessPointList.isEmpty() )
        {
            this.accessPointList.clear();
        } 

        if (!MD.getInterfaceList().isEmpty())
        {
            MD.getInterfaceList().clear();
        }

        if (!MD.getWirelessList().isEmpty())
        {
            MD.getWirelessList().clear();
        }
        
        if (!MD.getAccessPointList().isEmpty())
        {
            MD.getAccessPointList().clear();
        }        
        
        int i;
        
        this.interfaceList.addAll(Main.ai.getInterfaceList());
        
        for (i = 0; i < this.interfaceList.size(); i++)
        {
            server.InterfaceData sif = new server.InterfaceData();
            
            sif.setName(this.interfaceList.get(i).getName());
            sif.setMacAddress(this.interfaceList.get(i).getMac_address());
            sif.setIpAddress(this.interfaceList.get(i).getIp_address());
            sif.setSubnetmask(this.interfaceList.get(i).getSubnetmask());
            sif.setNetworkAddress(this.interfaceList.get(i).getNetwork_address());
            sif.setBroadcastAddress(this.interfaceList.get(i).getBroadcast_address());
            sif.setDefaultGateway(this.interfaceList.get(i).getDefault_gateway());
            sif.setMaxBitrate(this.interfaceList.get(i).getMax_bitrate());
            sif.setCurrentBitrate(this.interfaceList.get(i).getCurrent_bitrate());
            sif.setCurrentUsedrange(this.interfaceList.get(i).getCurrent_usedrange());
            sif.setPacketErrorRate(this.interfaceList.get(i).getPacket_error_rate());
            
            MD.getInterfaceList().add(sif);
        }
        
        this.wirelessList.addAll(Main.ai.getWirelessList());  
        
        for (i = 0; i < this.wirelessList.size(); i++)
        {
            server.WirelessData swif = new server.WirelessData();
            
            swif.setName(this.wirelessList.get(i).getName());
            swif.setMacAddress(this.wirelessList.get(i).getMac_address());
            swif.setIpAddress(this.wirelessList.get(i).getIp_address());
            swif.setSubnetmask(this.wirelessList.get(i).getSubnetmask());
            swif.setNetworkAddress(this.wirelessList.get(i).getNetwork_address());
            swif.setBroadcastAddress(this.wirelessList.get(i).getBroadcast_address());
            swif.setDefaultGateway(this.wirelessList.get(i).getDefault_gateway());
            swif.setMaxBitrate(this.wirelessList.get(i).getMax_bitrate());
            swif.setCurrentBitrate(this.wirelessList.get(i).getCurrent_bitrate());
            swif.setCurrentUsedrange(this.wirelessList.get(i).getCurrent_usedrange());
            swif.setPacketErrorRate(this.wirelessList.get(i).getPacket_error_rate());
            swif.setApMacAddress(this.wirelessList.get(i).getAp_mac_address());
            swif.setApEssid(this.wirelessList.get(i).getAp_essid());
            swif.setChannel(this.wirelessList.get(i).getChannel());
            swif.setApMode(this.wirelessList.get(i).getAp_mode());
            swif.setTransmitPower(this.wirelessList.get(i).getTransmit_power());
            swif.setLinkQuality(this.wirelessList.get(i).getLink_quality());
            swif.setSignalLevel(this.wirelessList.get(i).getSignal_level());
            swif.setNoiseLevel(this.wirelessList.get(i).getNoise_level());
            swif.setDiscardedPackets(this.wirelessList.get(i).getDiscarded_packets());
            
            MD.getWirelessList().add(swif);
        } 
        
        aap = (AvailableAccessPoints) Main.threadsTable.get("WirelessMonitor").getThread();
        this.accessPointList.addAll(aap.getAccessPointList());
        
        for (i = 0; i < this.accessPointList.size(); i++)
        {
            server.AccessPointData sap = new server.AccessPointData();
            
            sap.setMacAddress(this.accessPointList.get(i).getMac_address());
            sap.setEssid(this.accessPointList.get(i).getEssid());
            sap.setChannel(this.accessPointList.get(i).getChannel());
            sap.setMode(this.accessPointList.get(i).getMode());
            sap.setSignalLevel(this.accessPointList.get(i).getSignal_level()); 
            
            MD.getAccessPointList().add(sap);
        }        
        
        MD.setWifname(aap.getWifname());
    }
    
    synchronized private void sendMonitorData()
    {

        
        String device;
        try
        {
            device = InetAddress.getLocalHost().getHostName();
        } catch (UnknownHostException ex)
        {
            device = "user";
            Logger.getLogger(MonitorDataBuilder.class.getName()).log(Level.SEVERE, null, ex);
        }

        server.ServiceClient.getInstance().invokeSetMonitorData(device, this.MD, Main.prop.getProperty("service.ip"),
                Main.prop.getProperty("service.port"), Main.prop.getProperty("service.name"));        
    }
    
    /**Starts running this thread. */
    public void startThread()
    {
        this.start();
    }
    
    @Override
    public void run()
    {
        int s = Integer.parseInt(Main.prop.getProperty("sleeptime")), i;
        
        while ( true )
        {            
            try
            {
                sleep(s);
            } catch (InterruptedException ex)
            {
                Logger.getLogger(MonitorDataBuilder.class.getName()).log(Level.SEVERE, null, ex);
            }

            if (!Main.threadsTable.get("DataPrinter").getState())  //if it receives a termination signal 
            {
                break;
            }    
            
            this.buildMonitorData();
            
            this.sendMonitorData();
            
           if ( !Main.threadsTable.get("DataPrinter").getState() )  //if it receives a termination signal 
            {                   
                break;
            }  
           
        }
    }
}
