
package threadspack;

import datapack.WirelessData;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ListIterator;
import java.util.logging.Level;
import java.util.logging.Logger;
import mainpack.Main;

/**Thread that monitors the data of a specific wireless network interface. */
public class WifDataFinder extends Thread
{
    /**The name of the monitored wireless interface. */
    private String wifname;
    /**The WirelessData object of the monitored wireless interface. */
    private WirelessData winterf;
    /**IfDataFindr object to call the setInterface() method. */
    private IfDataFinder ifdf;

    /**Sets the wireless interface data.For every data that finds
     * stores them to the WirelessData object of this thread and if some 
     * data are not available stores them as "Not-available". */
    public synchronized void setWireless()
    {    
        String arg1 = this.wifname;    
        try
        {
            Process c = Runtime.getRuntime().exec("sudo iwconfig " + arg1);    //runs the iwconfig command
            try
            {
                c.waitFor();    //waits for the command to finish
            } catch (InterruptedException ex)
            {
                Logger.getLogger(IfDataFinder.class.getName()).log(Level.SEVERE, null, ex);
            }            
            BufferedReader in = new BufferedReader(new InputStreamReader(c.getInputStream()));
            String temps;

            temps = in.readLine();
            setApInfo(temps, in);

            temps = in.readLine();
            setTxpower(temps);

            setNetLinkInfo(temps, in);

            //calls the setInterface() method to set the data inherited by the InterfaceData
            ifdf.setWireless(arg1, this.winterf);   
            
            in.close();
        } catch (IOException ex)
        {
            Logger.getLogger(AvailableInterfaces.class.getName()).log(Level.SEVERE, null, ex);
            System.exit(1);
        }
    }

    private String convertChannel(String channel)
    {
        int ch = Integer.parseInt(channel.replace(".", ""));
        int convch = ((ch - 2412) / 5) + 1;
        return Integer.toString(convch);
    }

    private void setNetLinkInfo(String temps, BufferedReader in)
    {
        String[] sarray;
        
        try
        {
            while (temps != null)
            {
                if (temps.contains("Link Quality"))
                {
                    temps = temps.substring(temps.indexOf("Quality"));
                    sarray = temps.split("\\s+|=|:");
                    this.winterf.setLink_quality(sarray[1]);

                    if (temps.contains("Signal level"))
                    {
                        temps = temps.substring(temps.indexOf("Signal"));
                        sarray = temps.split("\\s+|:|=");
                        this.winterf.setSignal_level(sarray[2].concat(" dBm"));
                    }
                    else
                    {
                        this.winterf.setSignal_level("Not-available");
                    }
                    if (temps.contains("Noise level"))
                    {
                        temps = temps.substring(temps.indexOf("Noise"));
                        sarray = temps.split("\\s+|:|=");
                        this.winterf.setNoise_level(sarray[2].concat(" dBm"));
                    }
                    else
                    {
                        this.winterf.setNoise_level("Not-available");
                    }
                }
                if (temps.contains("Rx invalid"))
                {
                    int mpacks;

                    sarray = temps.split("\\s+|:");
                    mpacks = Integer.parseInt(sarray[4]) + Integer.parseInt(sarray[8]) + Integer.parseInt(sarray[12]);

                    temps = in.readLine();
                    sarray = temps.split("\\s+|:");
                    mpacks += Integer.parseInt(sarray[4]) + Integer.parseInt(sarray[7]);

                    this.winterf.setDiscarded_packets(Integer.toString(mpacks));
                }             
                temps = in.readLine();
            }
            if (this.winterf.getLink_quality() == null)
            {
                this.winterf.setLink_quality("Not-available");
                this.winterf.setSignal_level("Not-available");
                this.winterf.setNoise_level("Not-available");
            }
            if (this.winterf.getDiscarded_packets() == null)
            {
                this.winterf.setDiscarded_packets("Not-available");
            }
        } catch (IOException ex)
        {
            Logger.getLogger(WifDataFinder.class.getName()).log(Level.SEVERE, null, ex);
            System.exit(1);
        }
    }

    private void setTxpower(String temps)
    {
        String[] sarray;
        
        sarray = temps.split("\\s+|=|:");
        if (sarray[1].equals("Bit"))
        {
            if (sarray[4].startsWith("Mb"))
            {
                this.winterf.setMax_bitrate(Integer.toString(Integer.parseInt(sarray[3])*1000).concat(" kb/s"));

            }
            else
            {
                this.winterf.setMax_bitrate(sarray[3].concat(" kb/s"));
            }
            if (temps.contains("Tx-Power"))
            {
                this.winterf.setTransmit_power(sarray[6].concat(" dBm"));
            }
            else
            {
                this.winterf.setTransmit_power("Not-available");
            }
        }
        else if (sarray[1].equals("Tx-Power"))
        {
            this.winterf.setTransmit_power(sarray[2].concat(" dBm"));
        }
        else
        {
            this.winterf.setTransmit_power("Not-available");
        }
    }

    private void setApInfo(String temps, BufferedReader in)
    {
        String[] sarray;
        
        temps = temps.substring(temps.indexOf("ESSID:"));
        sarray = temps.split(":\"|\"\\s+|:");
        this.winterf.setAp_essid(sarray[1]);
        
        try
        {
            temps = in.readLine();
        } catch (IOException ex)
        {
            Logger.getLogger(WifDataFinder.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        temps = temps.substring(temps.indexOf("Mode"));
        sarray = temps.split("\\s+|:|=");
        this.winterf.setAp_mode(sarray[1]);
        
        if ( temps.contains("Frequency"))
        {
            temps = temps.substring(temps.indexOf("Frequency"));
            sarray = temps.split("\\s+|=|:");
            this.winterf.setChannel(convertChannel(sarray[1]));
        }
        else
        {
            this.winterf.setChannel("Not-available");
        }
        
        
        if (temps.contains("Point:"))
        {
            temps = temps.substring(temps.indexOf("Point:"));
            sarray = temps.split("\\s+|:\\s+");
            this.winterf.setAp_mac_address(sarray[1]);
        }
        else
        {
            sarray[1] = "Not-Associated";
            this.winterf.setAp_mac_address(sarray[1]);
            if (temps.contains("Bit"))
            {
                temps = temps.substring(temps.indexOf("Bit"));
                sarray = temps.split("\\s+|=|:");

                if (sarray[3].startsWith("Mb"))
                {
                    this.winterf.setMax_bitrate(Integer.toString(Integer.parseInt(sarray[2]) * 1000).concat(" kb/s"));

                }
                else
                {
                    this.winterf.setMax_bitrate(sarray[2].concat(" kb/s"));
                }
            }
            
        }
        
        if (sarray[1].equals("Not-Associated"))
        {
            this.winterf.setAp_essid("Not-connected");
        }
    }
    
    /**This method is used to print all data of the wireless interface that is being monitored by this thread.
     * @return all the wireless interface data to a string*/
    public synchronized String getDataString()
    {
        return this.winterf.toString();
    }      
        
    /**Sets the name and the WirelessData object of the wireless interface that this thread 
     * will monitor and then starts running the thread.
     * @param wifname the name of the wireless interface to be monitored
     * @param winterf the WirelessData object to be monitored*/
    public void startThread(String  wifname, WirelessData winterf)
    {
        this.wifname = wifname;
        this.winterf = winterf;
        this.ifdf = new IfDataFinder();
        this.start();
    }
    
    
    @Override
    public void run()
    {  
        long t;
        int s = 0;
        ListIterator<Integer> it = Main.markovList.listIterator();        

        WirelessData wifDataTemp;
        
        while ( true )
        {
            try
            {
                t = System.currentTimeMillis();
                
                wifDataTemp = (WirelessData) this.winterf.clone();  //clone the current InterfaceData object for future comparison                
                
                this.setWireless();
                
                t = System.currentTimeMillis() - t;

                //compares the old an the new InterfaceData objects 
                if ( !this.winterf.equals(wifDataTemp) )    //it had an event 
                {
                    s = Main.markovList.get(0);  //go to the first state
                    it = Main.markovList.listIterator(1);
                    System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+ this.getName()+
                    "] INFO WirelessDataFinder – Change on the wireless interface: "+this.wifname+". State= "+s+" Sleeptime= "+(t > s ? 0 : s - t)+"ms");                     
                }
                else
                {
                    if (it.hasNext())   //if it is not in the end of the markov list
                    {
                        s = it.next();  //go to the next node of the markov list 
                    }
                    System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+ this.getName()+
                    "] INFO WirelessDataFinder – No change on the wireless interface: "+this.wifname+". State= "+s+" Sleeptime= "+(t > s ? 0 : s - t)+"ms");                    
                }                 
                
                if ( !Main.threadsTable.get(wifname).getState() )   //if it receives a termination signal 
                {                    
                    break;
                }                               
               
               sleep( t > s ? 0 : s-t);
               
                if ( !Main.threadsTable.get(wifname).getState() )   //if it receives a termination signal 
                {                    
                    break;
                }                  
                
            } catch (InterruptedException ex)
            {
                Logger.getLogger(AvailableInterfaces.class.getName()).log(Level.SEVERE, null, ex);
                System.exit(1);
            }
        }
        System.out.println(System.currentTimeMillis()-Main.start_time+"\t["+this.getName()
                   +"] INFO AvailableInterfacesMonitor – Terminated InterfaceMonitor for: "+this.wifname); 
    }
}
