
package main;

import dbpack.DatabaseActions;
import guipack.UserInterface;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.ws.Endpoint;
import server.TestWebServiceImpl;


/**The class that contains the main method. */
public class Main
{
    /**The properties file that contains the initial setup info. */
    public static Properties props = new Properties();
    /**DatabaseActions singleton object that makes all the necessary actions between the service and the database,
     * contains the middle memory for updating the gui and the database and runs the thread that checks periodically
     * if any of the devices doesn't send data anymore. */
    public static DatabaseActions db;
    /**The UserInterface object that contains the graphical user interface. */
    public static UserInterface gui;
    /**Boolean variable that shows to the DataBaseActions thread when to stop. */
    public static boolean running = true;
    
    /**Main method that loads the properties from the property file,
    * creates the DatabaseActions object and starts the thread,
    * creates a UserInterface object and starts the graphical user interface,
    * and finally publishes the web service to the url provided by he properties file
    * This method also contains an anonymous thread that is added a shutdown hook.
    * @param args Argument given by the user when running the program that contains the path of the property file. */
    public static void main(String[] args)
    {
        FileInputStream in = null;
        
        if (args.length == 0)
        {
            System.out.println("Not enough arguments! The program will now exit...");
            System.exit(0);
        }        
        
        try
        {
            in = new FileInputStream(args[0]);
            props.load(in);
        }catch (FileNotFoundException ex) {

            Logger lgr = Logger.getLogger(Main.class.getName());
            lgr.log(Level.SEVERE, ex.getMessage(), ex);            

        } catch (IOException ex) {

            Logger lgr = Logger.getLogger(Main.class.getName());
            lgr.log(Level.SEVERE, ex.getMessage(), ex);

        } finally {
            
            try {
                 if (in != null) {
                     in.close();
                 }
            } catch (IOException ex) {
                Logger lgr = Logger.getLogger(Main.class.getName());
                lgr.log(Level.SEVERE, ex.getMessage(), ex);
            } 
        }  
        
        db = DatabaseActions.getInstance();          
        
        db.start(); //starts the DatabaseActions thread
        
        gui = new UserInterface();        
        
        Endpoint.publish("http://"+ props.getProperty("service.ip")
                                  +":"+props.getProperty("service.port")
                                  +"/"+props.getProperty("service.name"), 
                                   new TestWebServiceImpl());
        
        //anonymous thread added shutdown hook
        Runtime.getRuntime().addShutdownHook(new Thread()
        {
            @Override
            public void run()
            {
                System.out.println();
                runWhenShuttingDown();
            }
        });
      
    }
    
    
    private static void runWhenShuttingDown()
    { 
        running = false; //sends termination signal to the DatabaseActions thread
        try
        {
            db.join(); //waits for it to finish
        } catch (InterruptedException ex)
        {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
}
