package gr.askisi3.activities;

import gr.askisi3.data.AppData;
import gr.askisi3.data.TerminalData;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Color;
import android.os.BatteryManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.View;
import android.widget.TextView;
import android.widget.ToggleButton;

/**GPS Activity that contains the fields with the information about
 * the battery's level and charging state of the device and a button to enable
 * and disable the monitoring of this information. It also contains
 * the thread required for this task. */
public class BatteryActivity extends Activity
{
	/**The text view that shows the collected information about the battery. */	
	private TextView textBatteryInfo;
	/**The button for enabling and disabling the battery monitoring. */
	private ToggleButton button;	

	Handler handler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			// get the bundle and extract data by key
			Bundle b = msg.getData();
			String key = b.getString("My Key");
	    	textBatteryInfo.setText(key);			
		}
	}; 	
	
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_battery);		

		textBatteryInfo = (TextView) findViewById(R.id.battery_info);		
    	button = (ToggleButton) findViewById(R.id.button);		
		
    	if (AppData.getInstance().batThread) //if the battery thread is running
    	{
        	textBatteryInfo.setText("Battery Level: "+TerminalData.getInstance().getBatteryLevel()+" %\n\n"+
                    "Charging State: "+(TerminalData.getInstance().isCharging()? "Charging" : "Not Charging"));    		
        	((MyThread)AppData.getInstance().batBackground).setHandler(handler);
    		button.toggle();     		
    	}
    	else
    	{
        	textBatteryInfo.setText("Battery Level: Not-Monitoring\n\n"+
                    "Charging State: Not-Monitoring");    		
    	}    	
    	
    	if (AppData.getInstance().sendThread) //if the thread that sends data to the web-service is running
    	{
    		button.setClickable(false);
    		button.setBackgroundColor(Color.GRAY);
    	}
    	else
    	{
    		button.setClickable(true);
    	}    	
	}
    
    @Override
    protected void onStart() {
        // The activity is about to become visible.
        super.onStart();
    	if (AppData.getInstance().batThread)
    	{
    		((MyThread)AppData.getInstance().batBackground).setAtive(true);
    	}        						
    }
    
    @Override
    protected void onResume() {
        // The activity has become visible (it is now "resumed").    	
        super.onResume();
    	if (AppData.getInstance().batThread)
    	{
    		((MyThread)AppData.getInstance().batBackground).setAtive(true);
    	}  				
    }
    
    @Override
    protected void onPause() {
        // Another activity is taking focus (this activity is about to be "paused").    	
        super.onPause();
    	if (AppData.getInstance().batThread)
    	{
    		((MyThread)AppData.getInstance().batBackground).setAtive(false);
    	}  			
    }
    
    @Override
    protected void onStop() {
        // The activity is no longer visible (it is now "stopped")    	
        super.onStop();  
    	if (AppData.getInstance().batThread)
    	{
    		((MyThread)AppData.getInstance().batBackground).setAtive(false);
    	}  		
    }
       
    
    @Override
    protected void onDestroy() {
        // The activity is about to be destroyed.       	
        super.onDestroy();   	
    }
    
    /**Method that is called when the button Enable Monitoring is pushed. 
     * It starts the thread for monitoring the battery if it's not enabled 
     * or it stops this thread if it's enabled. */    
    public void buttonPushed(View view) 
    {
    	if (AppData.getInstance().batThread) //if the gps thread is running
    	{
    		//stops it 
    		AppData.getInstance().batThread = false;
    		
    		try
			{
    			AppData.getInstance().batBackground.join();
			} catch (InterruptedException e)
			{
				e.printStackTrace();
			}
    		AppData.getInstance().batBackground = null;
    		
        	this.textBatteryInfo.setText("Battery Level: Not-Monitoring\n\n"+
                    "Charging State: Not-Monitoring"); //refresh the gui info    		
    	}
    	else
    	{
    		//starts it
    		AppData.getInstance().batThread = true;
        	
    		AppData.getInstance().batBackground = new MyThread(handler);
    		AppData.getInstance().batBackground.start();        	
    	}
    }

    /**Thread that monitors the battery information about the level and charging
     * of the device and sends the data collected to the gui if the Battery
     * Activity is currently showing on the screen. */
    private class MyThread extends Thread 
    {  
    	private Handler mHandler;
    	private boolean active;

    	public MyThread(Handler handler){
    		super();
    		mHandler=handler;
    		active = true;
    	}

    	public void setHandler(Handler handler){
    		mHandler=handler;
    	}  
    	
    	public void setAtive(boolean act){
    		active = act;
    	}    	
    	
    	/**Method that contains a broadcast receiver for the battery and when
    	 * it's called runs the method onReceive finding the battery's info*/
	    private void batteryLevel() 
	    {
	        BroadcastReceiver batteryLevelReceiver = new BroadcastReceiver() 
	        {
	            public void onReceive(Context context, Intent intent) 
	            {
	                context.unregisterReceiver(this);
	                int rawlevel = intent.getIntExtra(BatteryManager.EXTRA_LEVEL, -1);
	                int scale = intent.getIntExtra(BatteryManager.EXTRA_SCALE, -1);
	                // Are we charging / charged?
	                int status = intent.getIntExtra(BatteryManager.EXTRA_STATUS, -1);
	                boolean isCharging = status == BatteryManager.BATTERY_STATUS_CHARGING ||
	                                     status == BatteryManager.BATTERY_STATUS_FULL;
	                int level = -1;
	                if (rawlevel >= 0 && scale > 0) 
	                {
	                    level = (rawlevel * 100) / scale;
	                }
	                //sets the new data to the Terminal Data object
	                TerminalData.getInstance().setBatteryLevel(level);
	                TerminalData.getInstance().setCharging(isCharging);
	            }
	        };
	        IntentFilter batteryLevelFilter = new IntentFilter(Intent.ACTION_BATTERY_CHANGED);
	        registerReceiver(batteryLevelReceiver, batteryLevelFilter);
	    }	    	
    	
		@Override
		public void run() 
		{  
			//loads sleep-time from properties
			int s = Integer.parseInt(AppData.getInstance().props.getProperty("sleeptime"));
			
			while (true)
			{	
				batteryLevel();	
				
				if(active) //if battery activity currently active 
				{     
					//sends the data to the handler
					Message msg = new Message();
					Bundle b = new Bundle();
					b.putString("My Key", "Battery Level: "+TerminalData.getInstance().getBatteryLevel()+" %\n\n"+
							"Charging State: "+(TerminalData.getInstance().isCharging()? "Charging" : "Not Charging"));				
					msg.setData(b);
					mHandler.sendMessage(msg);
				}
				
				if(!AppData.getInstance().batThread)
				{
					break;
				}
				
				try
				{
					sleep(s);
				} catch (InterruptedException e)
				{
					e.printStackTrace();
				}  
				
				if(!AppData.getInstance().batThread)
				{
					break;
				}    							
			}
		}
	}
}

