package gr.askisi3.activities;

import gr.askisi3.data.AppData;
import gr.askisi3.data.TerminalData;

import java.io.IOException;

import org.ksoap2.SoapEnvelope;
import org.ksoap2.serialization.PropertyInfo;
import org.ksoap2.serialization.SoapObject;
import org.ksoap2.serialization.SoapSerializationEnvelope;
import org.ksoap2.transport.HttpTransportSE;
import org.xmlpull.v1.XmlPullParserException;

import android.app.Activity;
import android.graphics.Color;
import android.os.Bundle;
import android.telephony.TelephonyManager;
import android.view.View;
import android.widget.TextView;
import android.widget.ToggleButton;

/**General Activity that contains the fields with the information about
 * the model, the name, and the version of the device and a button to enable
 * and disable sending this information to the web-service. It also contains
 * the thread required for this task. */
public class GeneralActivity extends Activity
{
	/**The text view that shows the collected information about the battery. */	
	private TextView textDeviceInfo;
	/**The button for enabling and disabling the battery monitoring. */	
	private ToggleButton button;	

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_general);		

		this.textDeviceInfo = (TextView) findViewById(R.id.device_info);
		
    	this.textDeviceInfo.setText("Device Model: "+TerminalData.getInstance().getModel()+"\n\n"+
    			                    "Device Manufacturer: "+TerminalData.getInstance().getManufacturer()+"\n\n"+
    	                            "Device SDK version: "+TerminalData.getInstance().getVersion());
    	
    	button = (ToggleButton) findViewById(R.id.button);
    	
    	//if the battery and gps threads are running
    	if (AppData.getInstance().gpsThread && AppData.getInstance().batThread) 
    	{
    		button.setClickable(true);
    	}
    	else
    	{
    		button.setClickable(false);
    		button.setBackgroundColor(Color.GRAY);
    	}
    	
    	if(AppData.getInstance().sendThread) //if the thread that sends data to the web-service is running
    	{
    		button.toggle();
    	}
	}
    
    @Override
    protected void onStart() {
        super.onStart();
        // The activity is about to become visible.
    }
    
    @Override
    protected void onResume() {
        super.onResume();
        // The activity has become visible (it is now "resumed").
    }
    
    @Override
    protected void onPause() {
        super.onPause();
        // Another activity is taking focus (this activity is about to be "paused").
    }
    
    @Override
    protected void onStop() {
        super.onStop(); 
        // The activity is no longer visible (it is now "stopped")
    }
    
    @Override
    protected void onDestroy() {
        super.onDestroy();
        // The activity is about to be destroyed.
    }    
    
    /**Method that is called when the button Start Sending Data is pushed. 
     * It starts the thread for sending the data collected if it's not enabled 
     * or it stops this thread if it's enabled. */   
    public void buttonPushed(View view)
    {    	
    	if (AppData.getInstance().sendThread) //if the send thread is running
    	{
    		//stops it 
    		AppData.getInstance().sendThread = false;
    		
    		try
			{
    			AppData.getInstance().sendBackground.join();
			} catch (InterruptedException e)
			{
				e.printStackTrace();
			}
    		AppData.getInstance().sendBackground = null;    		
    	}
    	else
    	{    
    		//starts it
    		AppData.getInstance().sendThread = true;
        	
    		AppData.getInstance().sendBackground = new MyThread();
    		AppData.getInstance().sendBackground.start();        	
    	}
    }
 
    /**Thread that makes the connection with the web-service and then sends to it
     * the collected information about the battery and gps of the device*/    
    private class MyThread extends Thread 
    {  
    	//loads sleep-time from properties
    	int s = Integer.parseInt(AppData.getInstance().props.getProperty("sleeptime"));
    	
    	String NAMESPACE = "http://server/";
    	String METHOD_NAME = "setTerminalData";
    	String SOAP_ACTION = "\"http://server/setTerminalData\"";
    	//loads connection settings of the web-service from properties to build the url
    	String URL = "http://"+AppData.getInstance().props.getProperty("service.ip")+
    			     ":"+AppData.getInstance().props.getProperty("service.port")+
    				 "/"+AppData.getInstance().props.getProperty("service.name")+"?WSDL";

		@Override
		public void run() 
		{  		
			String str, device;
			//gets the IMEI of the device
			device = ((TelephonyManager) getSystemService(TELEPHONY_SERVICE)).getDeviceId();
			
			while (true)
			{	
				//gets the data collected from the TrminalData Object to a string
				str = TerminalData.getInstance().toString();
				
				SoapObject request = new SoapObject(NAMESPACE, METHOD_NAME);
				
				PropertyInfo propInfo = new PropertyInfo();
				propInfo.name = "arg0"; 
				propInfo.setValue(device);
				propInfo.setType(String.class);
				request.addProperty(propInfo);
				
				PropertyInfo propData = new PropertyInfo();
				propData.name = "arg1"; 
				propData.setValue(str);
				propData.setType(String.class);
				request.addProperty(propData);				
				
				SoapSerializationEnvelope envelope = new SoapSerializationEnvelope(SoapEnvelope.VER11);
				envelope.setOutputSoapObject(request);
			 
				HttpTransportSE ht = new HttpTransportSE(URL);

				 try {
					ht.call(SOAP_ACTION, envelope);
					
				 } catch (IOException e) {
					e.printStackTrace();
				 } catch (XmlPullParserException e) {
					e.printStackTrace();
				 }				
				
				if(!AppData.getInstance().sendThread)
				{
					break;
				}
				
				try
				{
					sleep(s);
				} catch (InterruptedException e)
				{
					e.printStackTrace();
				}  
				
				if(!AppData.getInstance().sendThread)
				{
					break;
				}    							
			}
		}
	}
}
