package gr.askisi3.activities;

import gr.askisi3.data.AppData;
import gr.askisi3.data.TerminalData;
import android.app.Activity;
import android.app.Service;
import android.content.Context;
import android.graphics.Color;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.View;
import android.widget.TextView;
import android.widget.ToggleButton;

/**GPS Activity that contains the fields with the information about
 * the latitude and longitude of the device and a button to enable
 * and disable the monitoring of this information. It also contains
 * the thread required for this task. */
public class GpsActivity extends Activity
{
	/**The text view that shows the collected information about the location. */
	private TextView textGpsInfo;
	/**The button for enabling and disabling the gps monitoring. */
	private ToggleButton button;
	
	/**Handler for printing to the the text view of the gui the info that 
	 * the gps thread has collected*/
	Handler handler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			// get the bundle and extract data by key
			Bundle b = msg.getData();
			String key = b.getString("My Key");
	    	textGpsInfo.setText(key);			
		}
	}; 	
	
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{		
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_gps);			

		textGpsInfo = (TextView) findViewById(R.id.gps_info);	    	
    	button = (ToggleButton) findViewById(R.id.button);		
    	
    	if (AppData.getInstance().gpsThread) //if the the gps thread is running
    	{
        	this.textGpsInfo.setText("GPS Latitude: "+TerminalData.getInstance().getLatitude()+"\n\n"+
                    "GPS Longtitude: "+TerminalData.getInstance().getLongitude());     		
        	((MyThread)AppData.getInstance().gpsBackground).setHandler(handler);
    		button.toggle();     		
    	}
    	else
    	{
        	this.textGpsInfo.setText("GPS Latitude: Not-Monitoring\n\n"+
                    "GPS Longtitude: Not-Monitoring");    		
    	}
    	
    	if (AppData.getInstance().sendThread) //if the thread that sends data to the web-service is running
    	{
    		button.setClickable(false);
    		button.setBackgroundColor(Color.GRAY);
    	}
    	else
    	{
    		button.setClickable(true);
    	}	  
	}
    
    @Override
    protected void onStart() {
        // The activity is about to become visible.    	
        super.onStart();
    	if (AppData.getInstance().gpsThread)
    	{
    		((MyThread)AppData.getInstance().gpsBackground).setActive(true);
    	}        								
    }
    
    @Override
    protected void onResume() {
        // The activity has become visible (it is now "resumed").
        super.onResume();
    	if (AppData.getInstance().gpsThread)
    	{
    		((MyThread)AppData.getInstance().gpsBackground).setActive(true);
    	}  				               
    }
    
    @Override
    protected void onPause() {
        // Another activity is taking focus (this activity is about to be "paused").
        super.onPause();
    	if (AppData.getInstance().gpsThread)
    	{
    		((MyThread)AppData.getInstance().gpsBackground).setActive(false);
    	}  		
    }
    
    @Override
    protected void onStop() {
        // The activity is no longer visible (it is now "stopped")
        super.onStop();  
    	if (AppData.getInstance().gpsThread)
    	{
    		((MyThread)AppData.getInstance().gpsBackground).setActive(false);
    	}  		        
    }
         
    @Override
    protected void onDestroy() {
        // The activity is about to be destroyed.    	
        super.onDestroy();   	
    }
    
    /**Method that is called when the button Enable Monitoring is pushed. 
     * It starts the thread for monitoring the gps if it's not enabled 
     * or it stops this thread if it's enabled. */
    public void buttonPushed(View view)
    {
    	if (AppData.getInstance().gpsThread) //if the gps thread is running
    	{
    		//stops it   	
    		AppData.getInstance().gpsThread = false; 	    		    		
    		
    		try
			{
    			AppData.getInstance().gpsBackground.join();
			} catch (InterruptedException e)
			{
				e.printStackTrace();
			}
    		AppData.getInstance().gpsBackground = null;
    		
        	this.textGpsInfo.setText("GPS Latitude: Not-Monitoring\n\n"+
                    "GPS Longtitude: Not-Monitoring"); //refresh the gui info   		
    	}
    	else
    	{
    		//starts it
    		AppData.getInstance().gpsThread = true; 
        	
    		AppData.getInstance().gpsBackground = new MyThread(handler, this);
    		AppData.getInstance().gpsBackground.start();        	
    	}
    }  
    
    /**Thread that monitors the gps information about the latitude an longitude
     * of the device's location and sends the data collected to the gui if the GPS
     * Activity is currently showing on the screen. */
    private class MyThread extends Thread implements LocationListener 
    {  

    	private Handler mHandler;   	    	
    	private final Context mContext;
    	
    	private boolean active, isGPSEnabled, isNetworkEnabled;

    	/**The minimum distance to change Updates in meters. */
    	private static final long MIN_DISTANCE_CHANGE_FOR_UPDATES = 1; // 1 meter

    	/**The minimum time between updates in milliseconds. */
    	private static final long MIN_TIME_BW_UPDATES = 1000 * 1 * 1; // 1 second
    	
    	/**Default latitude loaded from the properties file. */
    	private double lat = Double.parseDouble(AppData.getInstance().props.getProperty("latitude"));
    	/**Default longitude loaded from the properties file. */
    	private double lon = Double.parseDouble(AppData.getInstance().props.getProperty("longitude"));    

    	/**A Location Manager. */
    	private LocationManager locationManager; 
    	
    	public MyThread(Handler handler, Context context){
    		super();
    		mHandler=handler;
    		active = true;
    		
    		this.mContext = context;
    		this.getLocation();    		
    	}
    	
    	public void setHandler(Handler handler){
    		mHandler=handler;
    	}  
    	
    	public void setActive(boolean act){
    		active = act;
    	}     	
    	
		@Override
		public void run() 
		{  	
			//loads sleep-time from properties
			int s = Integer.parseInt(AppData.getInstance().props.getProperty("sleeptime"));
			
			while (true)
			{
				this.getLocation(); 												
				
				if(active) //if gps activity currently active 
				{  
					//sends the data to the handler
					Message msg = new Message();
					Bundle b = new Bundle();
					b.putString("My Key", "GPS Latitude: "+TerminalData.getInstance().getLatitude()+"\n\n"+
		                    "GPS Longtitude: "+TerminalData.getInstance().getLongitude());
					msg.setData(b);
					mHandler.sendMessage(msg);
				}
				
				if(!AppData.getInstance().gpsThread)
				{
					break;
				}
				
				try
				{
					sleep(s);
				} catch (InterruptedException e)
				{
					e.printStackTrace();
				}  
				
				if(!AppData.getInstance().gpsThread)
				{
					break;
				}
			}
			
			if (locationManager != null) 
			{
				locationManager.removeUpdates(this); //stops using gps
			}
		}
		
		/**Method that checks if the device's gps or network provider is available
		 * for finding the current location and if at least one of them is, finds the info.
		 * Else it loads the default info from the properties file. */
		public void getLocation() 
		{			
			try {
				locationManager = (LocationManager) mContext.getSystemService(Service.LOCATION_SERVICE);

				// getting GPS status
				isGPSEnabled = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER);

				// getting network status
				isNetworkEnabled = locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER);

				if (!isGPSEnabled && !isNetworkEnabled) // gps and network provider isn't available
				{					
					TerminalData.getInstance().setLatitude(this.lat);
					TerminalData.getInstance().setLongitude(lon);					
				} 
				else 
				{	
					Location location = null;					
					if (isNetworkEnabled) // First gets location from Network Provider if it's available
					{
						locationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, MIN_TIME_BW_UPDATES,
								                               MIN_DISTANCE_CHANGE_FOR_UPDATES, this);
						if (locationManager != null) 
						{
							location = locationManager.getLastKnownLocation(LocationManager.NETWORK_PROVIDER);
							if (location != null) 
							{
								//sets the new data to the Terminal Data object
								TerminalData.getInstance().setLatitude(location.getLatitude());
								TerminalData.getInstance().setLongitude(location.getLongitude());
							}
							else
							{
								//sets the default data to the Terminal Data object
								TerminalData.getInstance().setLatitude(this.lat);
								TerminalData.getInstance().setLongitude(lon);
							}							
						}
					}				
					
					
					if (isGPSEnabled) // if GPS available gets lat/long using GPS
					{
						if (location == null) 
						{
							locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, MIN_TIME_BW_UPDATES,
									                               MIN_DISTANCE_CHANGE_FOR_UPDATES, this);
							if (locationManager != null) 
							{
								location = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
								if (location != null) 
								{
									//sets the new data to the Terminal Data object
									TerminalData.getInstance().setLatitude(location.getLatitude());
									TerminalData.getInstance().setLongitude(location.getLongitude());
								}
								else
								{
									//sets the default data to the Terminal Data object
									TerminalData.getInstance().setLatitude(this.lat);
									TerminalData.getInstance().setLongitude(lon);
								}
							}
						}
					}
				}

			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		@Override
		public void onLocationChanged(Location location) 
		{		
		}

		@Override
		public void onProviderDisabled(String provider) 
		{
		}

		@Override
		public void onProviderEnabled(String provider) 
		{
		}

		@Override
		public void onStatusChanged(String provider, int status, Bundle extras) 
		{
		}		
	}
}
