package gr.askisi3.activities;

import gr.askisi3.data.AppData;
import gr.askisi3.data.TerminalData;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import android.app.Activity;
import android.content.Intent;
import android.content.res.AssetManager;
import android.content.res.Resources;
import android.os.Bundle;
import android.view.View;

/**Main Activity of the program that contains the main menu
 * with the options for the user to choose and the methods 
 * that are called for each option.
 * On create it loads the required configurations from a property 
 * file located in the assets directory of the program. */
public class MainActivity extends Activity {	
	
	private TerminalData td;
	private AppData ad;

    @Override
    protected void onCreate(Bundle savedInstanceState) 
    {
        super.onCreate(savedInstanceState);        
        setContentView(R.layout.activity_main);  
        
        Resources resources = this.getResources();
        AssetManager assetManager = resources.getAssets();
        InputStream in = null;       
        
        td = TerminalData.getInstance();
        ad = AppData.getInstance();
        
        // Read from the /assets directory
        try {
            in = assetManager.open("config.properties");
            ad.props = new Properties();
            ad.props.load(in);         
        }catch (FileNotFoundException ex) {
        	ex.printStackTrace();

        } catch (IOException ex) {
        	ex.printStackTrace();

        } finally {
            
            try {
                 if (in != null) {
                     in.close();
                 }
            } catch (IOException ex) {
            	ex.printStackTrace();
            } 
        }         
    }
    
    @Override
    protected void onStart() {
        super.onStart();
        // The activity is about to become visible.
    }
    
    @Override
    protected void onResume() {
        super.onResume();
        // The activity has become visible (it is now "resumed").
    }
    
    @Override
    protected void onPause() {
        super.onPause();
        // Another activity is taking focus (this activity is about to be "paused").
    }
    
    @Override
    protected void onStop() {
        super.onStop();
        // The activity is no longer visible (it is now "stopped")
    }
    
    @Override
    protected void onDestroy() {
        // The activity is about to be destroyed.    	
        super.onDestroy();
        
		if( AppData.getInstance().gpsThread) //gps thread is running
		{        
			ad.gpsThread = false;
			try
			{
				AppData.getInstance().gpsBackground.join();
			} catch (InterruptedException e)
			{
				e.printStackTrace();
			}
			AppData.getInstance().gpsBackground = null;
		}
		
		if( AppData.getInstance().batThread) //battery thread is running
		{
			ad.batThread = false;
			try
			{
				AppData.getInstance().batBackground.join();
			} catch (InterruptedException e)
			{
				e.printStackTrace();
			}
			AppData.getInstance().batBackground = null;
		}
		
		if( AppData.getInstance().sendThread) //general thread is running
		{
			ad.sendThread = false;
			try
			{
				AppData.getInstance().sendBackground.join();
			} catch (InterruptedException e)
			{
				e.printStackTrace();
			}
			AppData.getInstance().sendBackground = null;
		}		
		
		td = null;
		ad = null;
		System.gc();
    }
    
    /**Method that is called when the General Info button is pushed.
     * It starts the General Info Activity.  */
    public void showDeviceInfo(View view)
    {
    	Intent intent = new Intent(this, GeneralActivity.class);
    	startActivity(intent);
    }
    
    /**Method that is called when the GPS Info button is pushed.
     * It starts the GPS Info Activity.  */
    public void showGpsInfo(View view)
    {
    	Intent gpsIntent = new Intent(this,GpsActivity.class);
    	startActivity(gpsIntent);    	    	
    }
    
    /**Method that is called when the Battery Info button is pushed. 
     * It starts the Battery Info Activity. */
    public void showBatteryInfo(View view)
    {
    	Intent batIntent = new Intent(this,BatteryActivity.class);
    	startActivity(batIntent);    	
    }
        
    
}
