package gr.askisi3.data;

/**Terminal data object that stores all the information collected 
 * for the device running this program. */
public class TerminalData
{
    /**The private instance of this singleton class. */
    private static TerminalData instance = null;	
	
	private String model;
	private String manufacturer;
	private String version;
	
	private double latitude;
	private double longitude;
	
	private int batteryLevel;
	private boolean charging;
	
    /**Private constructor that initiates the values. */	
	private TerminalData()
	{
		this.model = android.os.Build.MODEL;
		this.manufacturer = android.os.Build.MANUFACTURER;
		this.version = android.os.Build.VERSION.RELEASE;	
		
		this.latitude = 0.0;
		this.longitude = 0.0;
		
		this.batteryLevel = 0;
		this.charging = false;
	}
	
    /**Creates the unique object of this class if not created.
     * @return The unique instance to this singleton class*/
    public static synchronized TerminalData getInstance()
    {
        if (instance == null)
        {
            instance = new TerminalData();
        }
        return instance;
    }
    
    @Override
    public Object clone() throws CloneNotSupportedException
    {
        throw new CloneNotSupportedException();
    }	

	public String getModel()
	{
		return model;
	}

	public String getManufacturer()
	{
		return manufacturer;
	}

	public String getVersion()
	{
		return version;
	}

	public int getBatteryLevel()
	{
		return batteryLevel;
	}

	public boolean isCharging()
	{
		return charging;
	}

	public double getLatitude()
	{
		return latitude;
	}

	public double getLongitude()
	{
		return longitude;
	}

	public void setLatitude(double latitude)
	{
		this.latitude = latitude;
	}

	public void setLongitude(double longitude)
	{
		this.longitude = longitude;
	}

	public void setBatteryLevel(int batteryLevel)
	{
		this.batteryLevel = batteryLevel;
	}

	public void setCharging(boolean charging)
	{
		this.charging = charging;
	}
	
	@Override
	public String toString()
	{
		String temp;
		
		temp = this.model+":"+this.manufacturer+":"+this.version+":"+String.valueOf(this.latitude)+":"+String.valueOf(this.longitude)+
				":"+String.valueOf(this.batteryLevel)+":"+(this.isCharging()? "Charging" : "Not Charging");
		return temp;
	}
	
}
